<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE_AFL.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * @category    design
 * @package     default
 * @copyright   Copyright (c) 2010 Magento Inc. (http://www.magentocommerce.com)
 * @license     http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <title>Magento Installation Wizard</title>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>

    <link type="image/x-icon" href="<?php echo $this->baseUrl()?>/favicon.ico" rel="icon"/>
    <link type="image/x-icon" href="<?php echo $this->baseUrl()?>/favicon.ico" rel="shortcut icon"/>

    <script type="text/javascript" src="<?php echo $this->baseUrl()?>/js/prototype.js" ></script>

    <link rel="stylesheet" href="<?php echo $this->baseUrl()?>/skin/install/reset.css" type="text/css" media="all"/>
    <link rel="stylesheet" href="<?php echo $this->baseUrl()?>/skin/install/boxes.css" type="text/css" media="all"/>
    <link rel="stylesheet" href="<?php echo $this->baseUrl()?>/skin/install/clears.css" type="text/css" media="all"/>

    <!--[if IE]><link rel="stylesheet" href="<?php echo $this->baseUrl()?>/skin/install/iestyles.css" type="text/css" media="all"/><![endif]-->
    <!--[if lt IE 7]><link rel="stylesheet" href="<?php echo $this->baseUrl()?>/skin/install/ie7minus.css" type="text/css" media="all"/><![endif]-->
</head>

<body>
<!-- [start] header -->
    <div class="header">
        <div class="header-top-container">
            <div class="header-top">
                <h1 id="logo">
                    <a href="<?php echo $this->baseUrl()?>"><img src="<?php echo $this->baseUrl()?>/skin/install/images/<?php echo($this->get('channel_logo')); ?>.gif" alt="Magento"/></a>
                </h1>
            </div>
        </div>
            </div>
<!-- [end] header -->

        <!-- [start] middle -->
           <div class="middle-container">
        <div class="middle col-2-left-layout">



            <!-- [start] left -->

            <div class="col-left side-col">
                <div style="border:1px solid #ccc; background:#f6f6f6;">
    <h2 style="margin-bottom:0; border-bottom:1px solid #ccc; padding:4px 10px; color:#3c5974; font-size:1.4em;">Installation</h2>
<ol style="padding:10px; border-top:1px solid #fff;">
    <li>Welcome</li>
    <li>Validation</li>
    <li>Magento Connect Manager Deployment</li>
    <?php echo($this->get('channel_steps')); ?>
    <li style="color:green; font-weight:bold; ">Download</li>
    <li >License Agreement</li>
    <li >Localization</li>
    <li >Configuration</li>

    <li >Create Admin Account</li>
    <li >You're All Set!</li>
</ol>
</div>

<br/>
<p>
    Having trouble installing Magento?    Check out our <a href="http://www.magentocommerce.com/install" id="installation_guide_link">Installation Guide</a>
    <script type="text/javascript">
        $('installation_guide_link').target = "installation_guide";
    </script>

</p>
&nbsp;
            </div>
            <div id="main" class="col-main">
            <!-- [start] global messages -->
                            <!-- [end] global messages -->
            <!-- [start] content -->
