<?php
// Distributed by license from CardinalCommerce Corporation
/////////////////////////////////////////////////////////////////////////////////////////////
//  CardinalCommerce (http://www.cardinalcommerce.com)
//  CentinelErrors.php
//  Version 1.2 02/17/2005
//
//	Usage
//		The Error Numbers and Descriptions are centralized and referenced by the CentinelClient.php.
//
/////////////////////////////////////////////////////////////////////////////////////////////

	 define("CENTINEL_ERROR_CODE_8000", "8000");
     define("CENTINEL_ERROR_CODE_8000_DESC", "Protocol Not Recogonized, must be http:// or https://");
	 define("CENTINEL_ERROR_CODE_8010", "8010");
     define("CENTINEL_ERROR_CODE_8010_DESC", "Unable to Communicate with MAPS Server");
	 define("CENTINEL_ERROR_CODE_8020", "8020");
     define("CENTINEL_ERROR_CODE_8020_DESC", "Error Parsing XML Response");
	 define("CENTINEL_ERROR_CODE_8030", "8030");
     define("CENTINEL_ERROR_CODE_8030_DESC", "Communication Timeout Encountered");
