<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE_AFL.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * @category    design
 * @package     default
 * @copyright   Copyright (c) 2010 Magento Inc. (http://www.magentocommerce.com)
 * @license     http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
?>
            <!-- [end] content -->
            </div>
        </div>

            <!-- [end] center -->
    </div>
        <!-- [end] middle -->

        <!-- [start] footer -->
    <div class="footer-container">
        <div class="footer">
        <p class="legality">
            Help Us to Keep Magento Healthy - <a href="http://www.magentocommerce.com/bug-tracking" id="bug_tracking_link"><strong>Report All Bugs</strong></a> (Downloader ver. <?php echo Maged_Controller::getVersion();?>)<br/>

<script type="text/javascript">
    $('bug_tracking_link').target = "varien_external";
</script>
            Magento is a trademark of Magento, Inc. Copyright &copy; 2010 Magento Inc.</p>
        </div>
    </div>
        <!-- [end] footer -->
    </body>
</html>
